
#include <stdio.h>
#include <stdlib.h>
#include <kos.h>


#include <plx/matrix.h>
#include <plx/prim.h>
#include <plx/font.h>
#include <plx/sprite.h>
#include <plx/list.h>
#include <plx/dr.h>


#include <malloc.h>
#include "hw_stats.h"




#include "BIG_PVR.h"



pvr_init_params_t pvr_params = {
    { PVR_BINSIZE_16, PVR_BINSIZE_16, PVR_BINSIZE_16, PVR_BINSIZE_16, PVR_BINSIZE_16 },
    1024 * 1024
};


extern uint8 romdisk[];
KOS_INIT_ROMDISK(romdisk);


//==============================
union U_COLOR
{
	unsigned char bgra[4];
	unsigned int cl;
};

struct S_PVR_VERTEX_W {
	uint32  flags;              
	float   x;                  
	float   y;                  
	float   z;                 
	float   u;                 
	float   v;                 
	U_COLOR  color;               // brief Vertex color 
	//uint32  oargb;              // brief Vertex offset color
	float	w;
};


#define pvr_dr_target_custom(vtx_buf_ptr) \
	({ (vtx_buf_ptr) ^= 32; \
	(S_PVR_VERTEX_W*)(0xe0000000 | (vtx_buf_ptr)); \
})




//==============================
pvr_stats_t g_stats;



char frame_count = 0;

float x_offset, y_offset, z_offset;


plx_font_t * fnt;
plx_fcxt_t * fnt_cxt;
point_t w;


char g_str[50];
char right_pressed = 0, left_pressed = 0;
char NUM_BIGS = 1;

S_PVR_VERTEX_W trans_verts[obj_NumVertex];





void USER_INIT()
{
    fnt = plx_font_load("/rd/font.txf");
    
    fnt_cxt = plx_fcxt_create(fnt, PVR_LIST_TR_POLY);
	plx_fcxt_setsize (fnt_cxt, 15);
	
    plx_mat3d_init();
    plx_mat3d_mode(PLX_MAT_PROJECTION);
    plx_mat3d_identity();
    plx_mat3d_perspective(45.0f, 640.0f / 480.0f, 0.1f, 100.0f);
    plx_mat3d_mode(PLX_MAT_MODELVIEW);
}


void USER_PRE_RENDER_UPDATE()
{
}


void USER_POST_RENDER_UPDATE()
{
	pvr_get_stats(&g_stats);

	if (!frame_count)
	{
		print_ram_stats();
		print_VRAM_stats();
		sprintf (g_str, "FPS: %.2f", g_stats.frame_rate);
	}


	frame_count++;
	if (frame_count > 10)
		frame_count = 0;
}

void USER_ON_EXIT()
{
    if(fnt_cxt)
        plx_fcxt_destroy(fnt_cxt);

    if(fnt)
        plx_font_destroy(fnt);
}





void render()
{
	int i, j;
	static short ang = 1;
	char str2[10];

    pvr_poly_cxt_t cxt;
    pvr_poly_hdr_t hdr;

	volatile S_PVR_VERTEX_W *p_dr_vbuf;

	pvr_dr_state_t dr_state;
	pvr_dr_init (dr_state);

	char list = PVR_LIST_OP_POLY;
	
	pvr_poly_cxt_col (&cxt, list);
	cxt.gen.shading = PVR_SHADE_GOURAUD;
	cxt.gen.culling = PVR_CULLING_NONE;	
	
	if (ang > 360)
		ang = 0;
		
	x_offset = 11, y_offset = -9, z_offset = 19;	

    pvr_wait_ready();
    pvr_scene_begin();	

	pvr_list_begin(PVR_LIST_OP_POLY);

	for (int x = 0; x < NUM_BIGS; x++)
	{
		x_offset -= 3.0f;			
		if (  ((x % 5) == 0))
		{
			y_offset += 3.0f;
			x_offset = 8.0f;
		}
	
		
		plx_mat3d_push();
			plx_mat3d_translate (x_offset, y_offset, z_offset);

			plx_mat3d_rotate ((float)(ang++), 0, 1, 0);

		    plx_mat_identity();
		    plx_mat3d_apply(PLX_MAT_SCREENVIEW);
		    plx_mat3d_apply(PLX_MAT_PROJECTION);
		    plx_mat3d_apply(PLX_MAT_MODELVIEW);
		
		
			for (i = 0, j = 0; i < obj_NumVertex; i++, j+=3)
			{
				trans_verts[i].x = obj_Vertices[j];
				trans_verts[i].y = -obj_Vertices[j+1];
				trans_verts[i].z = obj_Vertices[j+2];
	
				mat_trans_single3 (trans_verts[i].x, trans_verts[i].y, trans_verts[i].z);
			}
		plx_mat3d_pop();
	
		

	    pvr_poly_compile (&hdr, &cxt);
	    pvr_prim(&hdr, sizeof(hdr));


		int vert_idx = 0;
		int strip_idx = 0;

		for(i = 0; i < obj_NumStrips; i++)
		{
			j = obj_StripLength[i];
			for (int k = 0; k < (j+2); k++, strip_idx++)
			{
				vert_idx = obj_Strips[strip_idx];
				p_dr_vbuf = pvr_dr_target_custom (dr_state);
				p_dr_vbuf->flags = PVR_CMD_VERTEX;

				if (k == (j+1))
					p_dr_vbuf->flags = PVR_CMD_VERTEX_EOL;
					
				p_dr_vbuf->x     = trans_verts[ vert_idx ].x;
				p_dr_vbuf->y     = trans_verts[ vert_idx ].y;
				p_dr_vbuf->z     = trans_verts[ vert_idx ].z;
				p_dr_vbuf->u     = 1.0f;
				p_dr_vbuf->v     = 1.0f;
				p_dr_vbuf->color.cl = obj_VertexColors[ vert_idx ];
				p_dr_vbuf->w = 1.0f;			
				pvr_dr_commit (p_dr_vbuf);
			}
		}
			
	}
	pvr_list_finish();



	w.x = 10.0f;
	w.y = 20.0f;
	w.z = 5.0f;
	pvr_list_begin(PVR_LIST_TR_POLY);
		plx_fcxt_begin(fnt_cxt);
		plx_fcxt_setpos_pnt(fnt_cxt, &w);
		plx_fcxt_draw(fnt_cxt, g_str);

		w.y = 40;
		plx_fcxt_setpos_pnt(fnt_cxt, &w);
		sprintf (str2, "NUM_BIGS: %i", NUM_BIGS);
		plx_fcxt_draw(fnt_cxt, str2);
		
		plx_fcxt_end(fnt_cxt);
	pvr_list_finish();
		
	pvr_scene_finish();
}



//--------------------------------

// MAIN

//---------------------------------
int main(int argc, char **argv) {
    maple_device_t *cont;
    cont_state_t *state;


    printf("\n\n************************\nMDL_VIEW_PVR starts...\n");

   
	pvr_init (&pvr_params);
	//pvr_set_bg_color (0.5f, 0.5f, 0.5f);
	pvr_set_bg_color (0, 0, 0);


	USER_INIT();

    while(1)
	{
		USER_PRE_RENDER_UPDATE();

		render();

		// INPUT
        cont = maple_enum_type(0, MAPLE_FUNC_CONTROLLER);

        state = (cont_state_t *)maple_dev_status(cont);

        if(!state) {
            printf("Error reading controller\n");
            break;
        }

        if(state->buttons & CONT_DPAD_LEFT)
		{
			if (!left_pressed && NUM_BIGS)
				NUM_BIGS--;
			
			left_pressed = 1;		
		}
		else
		{
			left_pressed = 0;				
		}


        if(state->buttons & CONT_DPAD_RIGHT)
		{
			if (!right_pressed)
				NUM_BIGS++;
			
			right_pressed = 1;		
		}
		else
		{
			right_pressed = 0;		
		}
		
		

		USER_POST_RENDER_UPDATE();
    }

	USER_ON_EXIT();

	printf("\n\nafter ON_EXIT()...\n");

    return 0;
}
